/*******************************************************************************
 *
 * File Name: ANNmain.c
 * Author: Richard Baxter
 * Project: Advanced Neural Network (ANN)
 * Project Version: 1l3j 2-Sept-08
 * Comments: TH = Test Harness
 *
 *******************************************************************************/

#include <ctime>
#include <cstdlib>	//for random number generation
#include <cmath>

#include <string>
#include <iostream>
#include <fstream>
using namespace std;


#include "LDglobalDefs.h"

#include "ANNFormation.h"
#include "ANNTraining.h"
#include "ANNParser.h"
#include "ANNXMLconversion.h"
#include "ANNsprite.h"

#include "ANNData.h"
#include "ANNmain.h"

#ifdef USE_LRRC
#include "LRRCgame.h"
#endif


NeuronContainer * firstInputNeuronInNetwork;
NeuronContainer * firstOutputNeuronInNetwork;

long numberOfInputNeurons;
long numberOfOutputNeurons;

bool numberOfInputAndOutputNeuronsSelected;
bool loadedExperienceDataset;
bool formedNetwork;

bool loadANetworkFromXML()
{
	bool result = true;

	//Neural Network initialisations
	firstInputNeuronInNetwork = new NeuronContainer();

	long numberOfInputNeuronsLoaded;
	long numberOfOutputNeuronsLoaded;

	string xmlFileName = NET_XML_FILE_NAME;

	firstOutputNeuronInNetwork = readNetXMLFileAndRecordFormationVariables(xmlFileName, firstInputNeuronInNetwork, &numberOfInputNeuronsLoaded, &numberOfOutputNeuronsLoaded);
	//check  number of input and number of output neurons

	numberOfInputNeurons = numberOfInputNeuronsLoaded;
	numberOfOutputNeurons = numberOfOutputNeuronsLoaded;

	formedNetwork = true;

	return result;
}

bool createANetwork()
{
	bool result = true;

	//Neural Network initialisations
	firstInputNeuronInNetwork = new NeuronContainer();

	#ifdef DEBUG
	cout << "************************************************************************************" << endl;
	cout << "******** DEBUG preprocessor definition enabled... tracing execution path...  *******" << endl;
	cout << "************************************************************************************" << endl;
	#endif

	int layerDivergenceType;
	double meanLayerDivergenceFactor;
	double probabilityNeuronConnectionWithPreviousLayerNeuron;
	double probabilityOfSubnetCreation;
	bool useSubnetDependentNumberOfLayers;
	long maxNumRecursiveSubnets;
	double numberOfLayers;
	double probabilityNeuronConnectionWithAllPreviousLayersNeurons;


	double answerAsDouble;
	long answerAsInt;
	char answerAsChar;
	string answerAsString;


	if(!loadedExperienceDataset)
	{
		cout <<	"\nEnter the number of input neurons (2, 3, 4, 5 [Default], ... etc):\n\n>> ";
		cin >> answerAsString;
		answerAsInt = long(atof(answerAsString.c_str()));
		numberOfInputNeurons = answerAsInt;

		cout <<	"\nEnter the number of output neurons (2, 3 [Default], 4, 5, ... etc):\n\n>> ";
		cin >> answerAsString;
		answerAsInt = long(atof(answerAsString.c_str()));
		numberOfOutputNeurons = answerAsInt;

		numberOfInputAndOutputNeuronsSelected = true;
	}


	cout <<	"\nEnter the number of top level Layers (2, 3 [Default], ... etc):\n\n>> ";
	cin >> answerAsString;
	answerAsInt = long(atof(answerAsString.c_str()));
	numberOfLayers = answerAsInt;
	if(numberOfLayers < 2)
	{
		numberOfLayers = 2;
	}


	cout <<	"\nDo you wish to specify advanced network structure properties? (y/n)\n\n>> ";
	cin >> answerAsString;
	answerAsChar = answerAsString[0];

	if(answerAsChar == 'y')
	{
		//non subnet/ANN specific parameters....
		cout <<	"\nEnter layer divergence type [Default = " << LAYER_DIVERGENCE_TYPE_LINEAR_CONVERGING << "]:\n\t1 (LINEAR_CONVERGING), \n\t2 (LAYER_DIVERGENCE_TYPE_LINEAR_DIVERGING_THEN_CONVERGING),\n\t3 (LAYER_DIVERGENCE_TYPE_NONLINEAR_DIVERGING_THEN_CONVERGING),\n\t4 (LAYER_DIVERGENCE_TYPE_LINEAR_DIVERGING),\n\t5 (LAYER_DIVERGENCE_TYPE_LINEAR_DIVERGING_SQUARE2D),\n\t6 (LAYER_DIVERGENCE_TYPE_LINEAR_DIVERGING_SQUARE2D_RADIALBIAS). \n\n>> ";
		cin >> answerAsString;
		answerAsInt = long(atof(answerAsString.c_str()));
		layerDivergenceType = answerAsInt;

		//TEST!
		if((layerDivergenceType == LAYER_DIVERGENCE_TYPE_LINEAR_DIVERGING_SQUARE2D) || (layerDivergenceType == LAYER_DIVERGENCE_TYPE_LINEAR_DIVERGING_SQUARE2D_RADIALBIAS))
		{
			numberOfInputNeurons = 16;
			numberOfOutputNeurons = 16;
		}

		cout <<	"\nEnter the mean layer divergence factor [Default = " << DEFAULT_MEAN_LAYER_DIVERGENCE_FACTOR << "]:\n\n>> ";
		cin >> answerAsString;
		answerAsDouble = atof(answerAsString.c_str());
		meanLayerDivergenceFactor = answerAsDouble;

		//cout << "DEBUG:" << meanLayerDivergenceFactor  << endl;

		cout <<	"\nEnter the probability of neuron having a connection with a previous layer neuron [Default = " << DEFAULT_PROBABILITY_NEURON_CONNECTION_WITH_PREVIOUS_LAYER_NEURON_ANNTH << "]:\n\n>> ";
		cin >> answerAsString;
		answerAsDouble = atof(answerAsString.c_str());
		probabilityNeuronConnectionWithPreviousLayerNeuron = answerAsDouble;


		cout <<	"\nEnter the probability of a neuron having a direct link with all previous layers neurons [Default = " << DEFAULT_PROBABILITY_NEURON_CONNECTION_WITH_ALL_PREVIOUS_LAYERS_NEURONS_ANNTH << "]:\n\n>> ";
		cin >> answerAsString;
		answerAsDouble = atof(answerAsString.c_str());
		probabilityNeuronConnectionWithAllPreviousLayersNeurons = answerAsDouble;


		//subnet/ANN specific parameters....
		cout <<	"\nDo you wish to create Subnets? (y/n) [Default = n] \n\n>> ";
		cin >> answerAsString;
		answerAsChar = answerAsString[0];

		if(answerAsChar == 'y')
		{
			cout <<	"\nEnter the maximum number of subnet layers you wish to create (1,2,3,4,5,... etc) [Default = " << DEFAULT_ANN_MAX_NUM_RECURSIVE_SUBNETS_ANNTH << "]:\n\n>> ";
			cin >> answerAsString;
			answerAsInt = long(atof(answerAsString.c_str()));
			maxNumRecursiveSubnets = answerAsInt;

			cout <<	"\nEnter the probability of subnet creation [Default = " << DEFAULT_PROB_OF_SUBNET_CREATION_ANNTH << "]:\n\n>> ";
			cin >> answerAsString;
			answerAsDouble = atof(answerAsString.c_str());
			probabilityOfSubnetCreation = answerAsDouble;

			cout <<	"\nDo you wish to use subnet dependant number of layers? (y/n). [Default = n]:\n\n>> ";
			cin >> answerAsString;
			answerAsChar = answerAsString[0];
			if(answerAsChar == 'y')
			{
				useSubnetDependentNumberOfLayers = true;
			}
			else
			{
				useSubnetDependentNumberOfLayers = false;
			}

			firstOutputNeuronInNetwork = formAdvancedNeuralNetwork(firstInputNeuronInNetwork, numberOfInputNeurons, numberOfOutputNeurons, useSubnetDependentNumberOfLayers, probabilityOfSubnetCreation, maxNumRecursiveSubnets, numberOfLayers, layerDivergenceType, meanLayerDivergenceFactor, probabilityNeuronConnectionWithPreviousLayerNeuron, probabilityNeuronConnectionWithAllPreviousLayersNeurons);

		}
		else
		{
			firstOutputNeuronInNetwork = formNeuralNet(firstInputNeuronInNetwork, numberOfInputNeurons, numberOfOutputNeurons, numberOfLayers, layerDivergenceType, meanLayerDivergenceFactor, probabilityNeuronConnectionWithPreviousLayerNeuron, probabilityNeuronConnectionWithAllPreviousLayersNeurons);
		}
	}
	else
	{
		firstOutputNeuronInNetwork = formNeuralNetWithOptimisedProperties(firstInputNeuronInNetwork, numberOfInputNeurons, numberOfOutputNeurons, numberOfLayers);
	}




	cout << "subnet Divergence Type = " << layerDivergenceType << endl;
	cout << "mean Subnet Divergence Factor = " << meanLayerDivergenceFactor << endl;
	cout << "probability Of Neuron Connection Exclusivity = " << probabilityNeuronConnectionWithPreviousLayerNeuron <<endl;
	cout << "Probability Of Subnet Creation = " << probabilityOfSubnetCreation <<endl;
	cout << "use Subnet Dependent Number Of Layers = " << useSubnetDependentNumberOfLayers << endl;
	cout << "Max Number Recursive Subnets = " << maxNumRecursiveSubnets <<endl;
	cout << "Number Of top level Layers = " << numberOfLayers << endl;


	formedNetwork = true;

	return result;
}






bool loadAnExperienceDataFile()
{
	bool result = true;

	//parse test harness Data set

	string nameOfExperiencesDataSetFile;
	char answerAsChar;
	string answerAsString;

	cout <<	"\nDo you wish to use the default experience dataset name? (y/n) [Default = y] \n\n>> ";
	cin >> answerAsString;
	answerAsChar = answerAsString[0];

	if(answerAsChar == 'y')
	{
		nameOfExperiencesDataSetFile = EXPERIENCE_DATASET_FILE_NAME_ANNTH;
	}
	else
	{
		cout <<	"\nEnter the name of the experience dataset:\n\n>> ";
		cin >> answerAsString;
		nameOfExperiencesDataSetFile = answerAsString;
	}

	cout << "nameOfExperiencesDataSetFile = " << nameOfExperiencesDataSetFile << endl;

	//doSomething();
	ANNTHparseTestDataFile(nameOfExperiencesDataSetFile);

	if(numberOfInputAndOutputNeuronsSelected)
	{
		if(numInputNeurons != numberOfInputNeurons)
		{
			cout << "Warning: number of input neurons previously selected do not match with data file. Please recreate neural network with correct number of output neurons" << endl;
		}
		if(numOutputNeurons != numberOfOutputNeurons)
		{
			cout << "Warning: number of output neurons previously selected do not match with data file. Please recreate neural network with correct number of output neurons" << endl;
		}

	}
	numberOfInputNeurons = numInputNeurons;
	numberOfOutputNeurons = numOutputNeurons;

	numberOfInputAndOutputNeuronsSelected = true;
	loadedExperienceDataset = true;

	return result;
}


bool trainANetwork()
{
	bool result = true;

	int maxFolds = MAX_NUM_FOLDS_ANNTH;		//maximum number of folds are same regardless of TEST_LEVEL_X_ANNTH

	if(!loadedExperienceDataset)
	{
		if(!loadAnExperienceDataFile())
		{
			result = false;
		}
	}

	if(formedNetwork)
	{
		trainNeuralNetwork(firstInputNeuronInNetwork, firstOutputNeuronInNetwork, numberOfInputNeurons, numberOfOutputNeurons, maxFolds, firstExperienceInDataSet, numExperiences);
	}
	else
	{
		cout << "Warning: cannot train network - no network has been formed" << endl;
	}
	return result;

}

bool outputANetworkToXML()
{
	bool result = true;

	cout << "xml file name = " << NET_XML_FILE_NAME << endl;

	if(formedNetwork)
	{
		if(!writeNetXMLFile(NET_XML_FILE_NAME, firstInputNeuronInNetwork))
		{
			result = false;
		}
	}
	else
	{
		cout << "Warning: cannot output network to xml - no network has been formed" << endl;
	}
	return result;
}



bool outputANetworkAsVectorGraphics()
{
	bool result = true;

	if(formedNetwork)
	{
		cout << "vector graphics file name without sprites = " << NEURAL_NETWORK_VISUALISATION_DAT_FILE_NAME_WITHOUT_SPRITES << endl;
		cout << "vector graphics file name with sprites = " << NEURAL_NETWORK_VISUALISATION_DAT_FILE_NAME_WITH_SPRITES << endl;

		ANNcreateNeuralNetworkSceneFilesWithAndWithoutSprites(NEURAL_NETWORK_VISUALISATION_DAT_FILE_NAME_WITHOUT_SPRITES, NEURAL_NETWORK_VISUALISATION_DAT_FILE_NAME_WITH_SPRITES, firstInputNeuronInNetwork);
	}
	else
	{
		cout << "Warning: cannot output network as vector graphics - no network has been formed" << endl;
	}

	return result;
}


bool mainUI()
{
	bool result = true;

	bool UIstatus = true;
	numberOfInputAndOutputNeuronsSelected = false;
	loadedExperienceDataset = false;
	formedNetwork = false;

	while(UIstatus == true)
	{
		int answerAsInt;
		char answerAsChar;
		string answerAsString;

		cout << "\n ---\n";
		cout << "Welecome To ANN Command Console (Version 1e1a or higher)\n";
		cout << " ---\n";
		cout << "Do you wish to \n";
		cout << " ---\n";
		cout << "1. Load a network from xml \n";
		cout << "2. Create a new network \n";
		cout << " --- \n";
		cout << "3. Load an experience dataset \n";
		cout << " --- \n";
		cout << "4. Train the network \n";
		cout << " --- \n";
		cout << "5. Output the network as xml \n";
		cout << "6. Output the network as vector graphics (.ldr) \n";
		cout << " --- \n";
#ifdef USE_LRRC
		cout << "7. Generate Experience Data Using LRRC \n";
		cout << " ---\n";
#endif
		cout << "0. Exit\n";
		cout << " ---\n";
		cout <<	"Enter Answer (0,1,2,3... etc):\n\n>> ";


	#ifdef DEBUG_IN_TEXTPAD
		cout << "option 7. automatically chosen; DEBUG_IN_TEXTPAD" << endl;
		executeLRRCfunctionsWithAI();
		exit(0);
	#else
		cin >> answerAsString;
		answerAsInt = long(atof(answerAsString.c_str()));
		if(answerAsInt == 1)
		{
			loadANetworkFromXML();
		}
		else if(answerAsInt == 2)
		{
			createANetwork();
		}
		else if(answerAsInt == 3)
		{
			loadAnExperienceDataFile();
		}
		else if(answerAsInt == 4)
		{
			trainANetwork();
		}
		else if(answerAsInt == 5)
		{
			outputANetworkToXML();
		}
		else if(answerAsInt == 6)
		{
			outputANetworkAsVectorGraphics();
		}
		#ifdef USE_LRRC
		else if(answerAsInt == 7)
		{
			executeLRRCfunctionsWithAI();
		}
		#endif
		else if(answerAsInt == 0)
		{
			UIstatus = false;
		}
		else
		{
			cout << "\nInvalid answerAsInt, please try another command\n" << endl;
		}

	#endif
	}

	return result;
}


int main()
{
	bool result = true;

	//srand(static_cast<unsigned>(time(0)));
	srand( (unsigned)time(NULL) );	//seeds randomness

	result = mainUI();

	return result;
}




