/*******************************************************************************
 *
 * File Name: LRRCTHgame.cpp
 * Author: Richard Baxter
 * Project: Lego Rules CG Rounds Checker
 * Project Version: 1l3j 2-Sept-08
 * Project First Internal Release: 1aXx 18-Sept-05 (C)
 * Project Second Internal Release: 2aXx 02-April-06 (convert to C++)
 * Project Third Internal Release: 2b7d 26-Sept-06 (added sprites)
 * Project Fourth Internal Release: 2c4a 25-Feb-07 [rotation of models supported, copyCombatRelevantInheritedImmediateParentUnitDetailsIntoChildObject() no longer used]
 * Project Fifth Internal Release: 2d11a 3-June-07
 *
 *******************************************************************************/

#include "LDglobalDefs.h"

#include "THLRRCgame.h"
#include "LRRCgame.h"
#include "LRRCmovement.h"
#include "LRRCcombat.h"
#include "LRRCsprite.h"
#include "LDparser.h"
#include "LDreferenceManipulation.h"
#include "LRRCgameReferenceManipulation.h"
#include "XMLrulesClass.h"

#ifdef USE_ANN
#include "ANNneuronClass.h"
#include "ANNFormation.h"
#include "LRRCgameAI.h"
#include "LRRCunitClass.h"
#include "ANNTraining.h"
#include "ANNXMLconversion.h"
#include "ANNsprite.h"
#include "ANNUpdateAlgorithm.h"
#endif

#include <string>
#include <iostream>
#include <fstream>
#include <time.h>
#include <math.h>
using namespace std;






//Test Harness Code:
#ifdef USE_ANN

bool cullUnitList(UnitListClass * firstUnitInUnitList, int numberOfUnitsRequired)
{
	UnitListClass * currentUnitInUnitList = firstUnitInUnitList;
	
	for(int i=0; i<numberOfUnitsRequired; i++)
	{
		currentUnitInUnitList = currentUnitInUnitList->next;
	}
	currentUnitInUnitList->next = NULL;
}

bool randomiseUnitList(UnitListClass * firstUnitInNonRandomisedUnitList, UnitListClass * firstUnitInUnitList, long numberOfUnits)
{
	UnitListClass * currentUnitInNonRandomisedUnitList = firstUnitInNonRandomisedUnitList->next;
	UnitListClass * previousUnitInNonRandomisedUnitList = firstUnitInNonRandomisedUnitList;
	UnitListClass * currentUnitInUnitList = firstUnitInUnitList;
	
	long indexOfCurrentUnitInNonRandomisedUnitList = 0;
	long currentNumberOfUnitsInRandomisedUnitList = numberOfUnits;
	double randomNumberBetween0And1 = ((double(abs((short)rand())))/(ABS_OF_SHORT));
	long randomUnitSelected = (long)(randomNumberBetween0And1*(double)currentNumberOfUnitsInRandomisedUnitList);
	//cout << "randomUnitSelected = " << randomUnitSelected << endl;
		
	while(currentNumberOfUnitsInRandomisedUnitList != 0)		//(currentNumberOfUnitsInRandomisedUnitList != 0) or currentUnitInNonRandomisedUnitList->next != NULL)	//or (currentNumberOfUnitsInRandomisedUnitList != 0)
	{	
		/*
		if(currentUnitInNonRandomisedUnitList->next == NULL)
		{
			cout << "(currentUnitInNonRandomisedUnitList->next == NULL)" << endl;
		}
		*/
		
		//cout << "0" << endl;
		randomNumberBetween0And1 = ((double(abs((short)rand())))/(ABS_OF_SHORT));
		randomUnitSelected = (long)(randomNumberBetween0And1*(double)currentNumberOfUnitsInRandomisedUnitList);
		//cout << "randomUnitSelected = " << randomUnitSelected << endl;
		
		/*
		if(randomUnitSelected == (currentNumberOfUnitsInRandomisedUnitList-1))
		{
			exit(0);
		}
		*/
					 
		if(indexOfCurrentUnitInNonRandomisedUnitList == randomUnitSelected)
		{	
			//cout << "1" << endl;
			
			UnitListClass * newUnit = new UnitListClass();
			newUnit->number = currentUnitInNonRandomisedUnitList->number;
			ModelDetails * newUnitDetails = new ModelDetails();
			newUnit->unitDetails = newUnitDetails;
			copyAllUnitDetails(newUnit->unitDetails, currentUnitInNonRandomisedUnitList->unitDetails);
			currentUnitInUnitList->next = newUnit;
			currentUnitInUnitList = currentUnitInUnitList->next;
			
			//remove currentUnitInNonRandomisedUnitList from possible units
			previousUnitInNonRandomisedUnitList->next = currentUnitInNonRandomisedUnitList->next;
			
			if(indexOfCurrentUnitInNonRandomisedUnitList == 0)
			{
				firstUnitInNonRandomisedUnitList->next = currentUnitInNonRandomisedUnitList->next;
			}
			
			currentNumberOfUnitsInRandomisedUnitList--;
		
			//cout << "2" << endl;
		}
		else
		{
			previousUnitInNonRandomisedUnitList = previousUnitInNonRandomisedUnitList->next;
		}
		currentUnitInNonRandomisedUnitList = currentUnitInNonRandomisedUnitList->next;	
		indexOfCurrentUnitInNonRandomisedUnitList++;
		//cout << "indexOfCurrentUnitInNonRandomisedUnitList = " << indexOfCurrentUnitInNonRandomisedUnitList << endl;
		
		if(currentUnitInNonRandomisedUnitList->next == NULL)	//indexOfCurrentUnitInNonRandomisedUnitList == currentNumberOfUnits
		{//go back to start of loop...
			//cout << "here" << endl;
			indexOfCurrentUnitInNonRandomisedUnitList = 0;
			currentUnitInNonRandomisedUnitList = firstUnitInNonRandomisedUnitList->next;
			previousUnitInNonRandomisedUnitList = firstUnitInNonRandomisedUnitList;
		}
		//cout << "4" << endl;
		
	}
	currentUnitInUnitList->next = NULL;
}
	
int THtestANNusingCombatExperiences()
{
	//************************	
	//PART 0: Standard Initialisations
	//************************
	
	bool result = true;

	if(!parseLRRCRulesXMLFile())
	{
		result = false;
	}

	fillInCombatExternVariables();
	fillInGameExternVariables();
	fillInModelClassExternVariables();
	fillInParserExternVariables();
	fillInPlayerClassExternVariables();
	fillInLRRCSpriteExternVariables();

	Player * currentPlayer = new Player();

	#define IRRELEVANT (0)

	//************************
	//PART 1: Standard Initialisations; create neural network 1 (for unit properties and unit combat decisions) 
	//************************
	
	Reference *tempUnitReference1 = new Reference(true);
	Reference * tempUnitRefenceOpponent1 = new Reference(true);
	long tempUnitDecision1 = IRRELEVANT;
	Experience * tempExperience1 = new Experience;
	generateExperienceFromUnitPropertiesDecision(tempUnitReference1, tempUnitRefenceOpponent1, tempUnitDecision1, tempExperience1);
	long numberOfInputNeurons1;
	long numberOfOutputNeurons1;
	long numberOfLayers1;
	numberOfInputNeurons1 = countNumberOfExperienceInputs(tempExperience1);
	delete tempUnitReference1;
	delete tempUnitRefenceOpponent1;
	delete tempExperience1;
	numberOfOutputNeurons1 = COMBAT_EXPERIENCE_NUMBER_OF_DIFFERENT_OUTPUT_DECISIONS;
	NeuronContainer * firstInputNeuronInNetwork1 = new NeuronContainer();
	NeuronContainer * firstOutputNeuronInNetwork1;
	numberOfLayers1 = TH_LRRC_GAME_NUMBER_OF_NN_LAYERS; //(numberOfInputNeurons+numberOfOutputNeurons)/2;	//arbitrary NN number of layers heuristic - need to optimise this

	firstOutputNeuronInNetwork1 = formNeuralNetWithOptimisedProperties(firstInputNeuronInNetwork1, numberOfInputNeurons1, numberOfOutputNeurons1, numberOfLayers1);

	//create neural network 2 (for unit/group targetting/combat decisions) 

	Reference *tempUnitReference2 = new Reference(true);
	Reference * tempUnitRefenceOpponent2 = new Reference(true);
	long tempUnitDecision2 = IRRELEVANT;
	Experience * tempExperience2 = new Experience;
	generateExperienceFromUnitCombatDecision(tempUnitReference2, tempUnitRefenceOpponent2, tempUnitDecision2, tempExperience2);
	long numberOfInputNeurons2;
	long numberOfOutputNeurons2;
	long numberOfLayers2;
	numberOfInputNeurons2 = countNumberOfExperienceInputs(tempExperience2);
	delete tempUnitReference2;
	delete tempUnitRefenceOpponent2;
	delete tempExperience2;
	numberOfOutputNeurons2 = COMBAT_EXPERIENCE_NUMBER_OF_DIFFERENT_OUTPUT_DECISIONS;
	NeuronContainer * firstInputNeuronInNetwork2 = new NeuronContainer();
	NeuronContainer * firstOutputNeuronInNetwork2;
	numberOfLayers2 = TH_LRRC_GAME_NUMBER_OF_NN_LAYERS; //(numberOfInputNeurons+numberOfOutputNeurons)/2;	//arbitrary NN number of layers heuristic - need to optimise this

	firstOutputNeuronInNetwork2 = formNeuralNetWithOptimisedProperties(firstInputNeuronInNetwork2, numberOfInputNeurons2, numberOfOutputNeurons2, numberOfLayers2);

	/*
	//create neural network 3 (for global targetting/combat decisions) 

	Reference * tempUnitReference3 = new Reference(true);
	Reference * tempUnitRefenceOpponent3 = new Reference(true);
	Experience * tempExperience3 = new Experience;
	UnitListClass * tempUnit3 = new UnitListClass();
	Reference * tempReferenceInNULLSceneFile3;
	generateExperienceFromGlobalDecision(tempUnit3, tempReferenceInNULLSceneFile3, tempUnitReference3, tempUnitRefenceOpponent3, tempExperience3);
	double numberOfLayers3 = TH_LRRC_GAME_NUMBER_OF_NN_LAYERS-1;		//arbitrary NN number of layers heuristic - need to optimise this
	long numberOfInputNeurons3;
	long numberOfOutputNeurons3;
	int layerDivergenceType3 = LAYER_DIVERGENCE_TYPE_LINEAR_DIVERGING_SQUARE2D_RADIALBIAS;
	double meanLayerDivergenceFactor3 = 1.5;
	double probabilityNeuronConnectionWithPreviousLayerNeuron3 = 1.0;
	double probabilityNeuronConnectionWithAllPreviousLayersNeurons3 = 0.5;
	numberOfInputNeurons3 = countNumberOfExperienceInputs(tempExperience3);	//GLOBAL_EXPERIENCE_NUMBER_OF_DIFFERENT_INPUT_DECISIONS
	delete tempUnitReference3;
	delete tempUnitRefenceOpponent3;
	delete tempExperience3;
	numberOfOutputNeurons3 = GLOBAL_EXPERIENCE_NUMBER_OF_DIFFERENT_OUTPUT_DECISIONS;
	NeuronContainer * firstInputNeuronInNetwork3 = new NeuronContainer();
	NeuronContainer * firstOutputNeuronInNetwork3;

	firstOutputNeuronInNetwork3 = formNeuralNet(firstInputNeuronInNetwork3, numberOfInputNeurons3, numberOfOutputNeurons3, numberOfLayers3, layerDivergenceType3, meanLayerDivergenceFactor3, probabilityNeuronConnectionWithPreviousLayerNeuron3, probabilityNeuronConnectionWithAllPreviousLayersNeurons3);
	*/
	
	cout << "Game Neural Network Properties:" << endl;
	cout << "numberOfInputNeurons1 = " << numberOfInputNeurons1 << endl;
	cout << "numberOfOutputNeurons1 = " << numberOfOutputNeurons1 << endl;
	cout << "numberOfLayers1 = " << numberOfLayers1 << endl;
	cout << "numberOfInputNeurons2 = " << numberOfInputNeurons2 << endl;
	cout << "numberOfOutputNeurons2 = " << numberOfOutputNeurons2 << endl;
	cout << "numberOfLayers2 = " << numberOfLayers2 << endl;
	/*
	cout << "numberOfInputNeurons3 = " << numberOfInputNeurons3 << endl;
	cout << "numberOfOutputNeurons3 = " << numberOfOutputNeurons3 << endl;
	cout << "numberOfLayers3 = " << numberOfLayers3 << endl;
	*/

										
	currentPlayer->firstInputNeuronInNetwork[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN] = firstInputNeuronInNetwork1;
	currentPlayer->firstOutputNeuronInNetwork[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN] = firstOutputNeuronInNetwork1;
	currentPlayer->numberOfInputNeurons[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN] = numberOfInputNeurons1;
	currentPlayer->numberOfOutputNeurons[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN] = numberOfOutputNeurons1;
	currentPlayer->firstInputNeuronInNetwork[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN] = firstInputNeuronInNetwork2;
	currentPlayer->firstOutputNeuronInNetwork[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN] = firstOutputNeuronInNetwork2;
	currentPlayer->numberOfInputNeurons[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN] = numberOfInputNeurons2;
	currentPlayer->numberOfOutputNeurons[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN] = numberOfOutputNeurons2;
	/*
	currentPlayer->firstInputNeuronInNetwork[GAME_INDEX_OF_GLOBAL_EXPERIENCE_NN] = firstInputNeuronInNetwork3;
	currentPlayer->firstOutputNeuronInNetwork[GAME_INDEX_OF_GLOBAL_EXPERIENCE_NN] = firstOutputNeuronInNetwork3;
	currentPlayer->numberOfInputNeurons[GAME_INDEX_OF_GLOBAL_EXPERIENCE_NN] = numberOfInputNeurons3;
	currentPlayer->numberOfOutputNeurons[GAME_INDEX_OF_GLOBAL_EXPERIENCE_NN] = numberOfOutputNeurons3;			
	*/	

	
	//************************
	//PART 2: Generate a list of Units with Random Unit Properties
	//************************
	
	UnitListClass * firstUnitInNonRandomisedUnitList = new UnitListClass();
	
	
	int numberOfUnitsBeforeCull = THgenerateUnitListwithAllUnitProperties(firstUnitInNonRandomisedUnitList);

	UnitListClass * firstUnitInUnitList = new UnitListClass();
	firstUnitInUnitList->number = 0;
	ModelDetails * newUnitDetails = new ModelDetails();
	firstUnitInUnitList->unitDetails = newUnitDetails;
	copyAllUnitDetails(firstUnitInUnitList->unitDetails, firstUnitInNonRandomisedUnitList->unitDetails);

	randomiseUnitList(firstUnitInNonRandomisedUnitList, firstUnitInUnitList, (numberOfUnitsBeforeCull-1));
	
	int numberOfUnits = TH_LRRC_GAME_NUMBER_OF_UNITS_TO_FEED;
	cullUnitList(firstUnitInUnitList, numberOfUnits);
	
	/*
	//temp - print out unit list
	//int uin = 0;
	UnitListClass * currentUnitInList = firstUnitInUnitList;
	while(currentUnitInList->next != NULL)
	{
		//uin++;
		
		cout << "currentUnitInList->number = " << currentUnitInList->number << endl;
		
		//cout << "currentUnitInList->closeCombatAttackValue = " << currentUnitInList->unitDetails->closeCombatAttackValue << endl;
		//cout << "currentUnitInList->longDistanceAttackValue = " << currentUnitInList->unitDetails->longDistanceAttackValue << endl;
		//cout << "currentUnitInList->closeCombatAttackTotal = " << currentUnitInList->unitDetails->closeCombatAttackTotal << endl;
		//cout << "currentUnitInList->longDistanceAttackTotal = " << currentUnitInList->unitDetails->longDistanceAttackTotal << endl;
		//cout << "currentUnitInList->defenceTotal = " << currentUnitInList->unitDetails->defenceTotal << endl;
		
						
		currentUnitInList = currentUnitInList->next;
	}
	//cout << "uin = " << uin << endl;
	*/
						
	//************************
	//PART 3: perform Generic Combat With All Permutations Of Units InList And Add Experiences
	//************************
	
	//Perform testing over the following neural networks [NNBeingTested] and scenarios [currentPhase];
	int numberOfNetworks = 1;
	int numberOfScenarios = 1;
	int currentPhase;
	int NNBeingTested;

	currentPhase = GAME_PHASE_CLOSECOMBAT;
	NNBeingTested = GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN;		
	THperformGenericCombatWithAllPermutationsOfUnitsInListAndAddExperiences(currentPhase, firstUnitInUnitList, currentPlayer, NNBeingTested);
	
	currentPhase = GAME_PHASE_CLOSECOMBAT;
	NNBeingTested = GAME_INDEX_OF_COMBAT_EXPERIENCE_NN;
	THperformGenericCombatWithAllPermutationsOfUnitsInListAndAddExperiences(currentPhase, firstUnitInUnitList, currentPlayer, NNBeingTested);
	
	/*
	currentPhase = GAME_PHASE_LONGDISTANCECOMBAT;
	NNBeingTested = GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN;	
	THperformGenericCombatWithAllPermutationsOfUnitsInListAndAddExperiences(currentPhase, firstUnitInUnitList, currentPlayer, NNBeingTested);
		
	currentPhase = GAME_PHASE_LONGDISTANCECOMBAT;
	NNBeingTested = GAME_INDEX_OF_COMBAT_EXPERIENCE_NN;
	THperformGenericCombatWithAllPermutationsOfUnitsInListAndAddExperiences(currentPhase, firstUnitInUnitList, currentPlayer, NNBeingTested);
	*/
	
	//************************
	//PART 4: feed neural network with all experiences
	//************************

	if(numberOfUnits <= 100)	//for debugging speed, do not generate large files 
	{
		
		int currentGame = 1;
		for(int nn=0; nn < 2; nn++)		//by default this is GAME_NUMBER_OF_EXPERIENCE_NN (3), ie, do not currently use global experiences, generateExperienceFromGlobalDecision
		{
			#ifdef HEAPS_OF_RAM
			feedNeuralNetworkWithGameUnitExperiences(currentPlayer->firstInputNeuronInNetwork[nn], currentPlayer->firstOutputNeuronInNetwork[nn], currentPlayer->numberOfInputNeurons[nn], currentPlayer->numberOfOutputNeurons[nn], currentPlayer->firstUnitInUnitList, nn);
				//this is now done dynamically if do not have heaps of RAM
			#endif
	
	
			//cout << "H4" << endl;
	
			//	- For some reason this doesnt work... why???
			//now output the network to xml file
			string * XMLNNSceneFileName = new string();
			generateXMLNNSceneFileName(currentGame, XMLNNSceneFileName, nn);
			cout << "XMLNNSceneFileName = " << *XMLNNSceneFileName << endl;
	
			//cout << "currentPlayer->firstInputNeuronInNetwork[nn]->neuron->id = " << currentPlayer->firstInputNeuronInNetwork[nn]->neuron->id << endl; 
			if(!writeNetXMLFile(*XMLNNSceneFileName, currentPlayer->firstInputNeuronInNetwork[nn]))
			{
				result = false;
			}
			
				
			//cout << "H5" << endl;
	
			//now output the network to vector graphics file
			string * vectorGraphicsNNSceneFileName = new string();
			generateVectorGraphicsNNSceneFileName(currentGame, vectorGraphicsNNSceneFileName, nn);
			cout << "vectorGraphicsNNSceneFileName = " << *vectorGraphicsNNSceneFileName << endl;
	
			char * charstar = new char[(*vectorGraphicsNNSceneFileName).size()+1];
			const char * constcharstar = (*vectorGraphicsNNSceneFileName).c_str();
			strcpy(charstar, constcharstar);
	
			ANNcreateNeuralNetworkSceneFilesWithAndWithoutSprites(charstar, NULL, currentPlayer->firstInputNeuronInNetwork[nn]);
	
			//cout << "H6" << endl;
	
			#ifdef HEAPS_OF_RAM
			//now output the experiences to experience data set file
			string * experienceNNSceneFileName = new string();
			generateExperiencesNNSceneFileName(currentGame, experienceNNSceneFileName, nn);
			cout << "experienceNNSceneFileName = " << *experienceNNSceneFileName << endl;
	
			ofstream experienceDataSetOFStreamObject(experienceNNSceneFileName->c_str());	//experienceDataSetOFStreamObject
			addPlayerUnitExperiencesToOFStream(&experienceDataSetOFStreamObject, currentPlayer->firstUnitInUnitList, currentPlayer->id, nn);
			experienceDataSetOFStreamObject.close();
			#endif
			
			//cout << "H7" << endl;
		}
	}

	//************************
	//PART 5: Now Test Performance of Network; perform Generic Combat With All Permutations Of Units In List And Compare Experiences With NN
	//************************
	
	//Now Test Performance of Network;
	
	double totalErrorPropertiesExperience = 0;
	double totalErrorCombatExperience = 0;
	
	currentPhase = GAME_PHASE_CLOSECOMBAT;
	NNBeingTested = GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN;		
	totalErrorPropertiesExperience = THperformGenericCombatWithAllPermutationsOfUnitsInListAndCompareExperiencesWithNN(currentPhase, firstUnitInUnitList, currentPlayer, NNBeingTested);
	
	currentPhase = GAME_PHASE_CLOSECOMBAT;
	NNBeingTested = GAME_INDEX_OF_COMBAT_EXPERIENCE_NN;
	totalErrorCombatExperience = THperformGenericCombatWithAllPermutationsOfUnitsInListAndCompareExperiencesWithNN(currentPhase, firstUnitInUnitList, currentPlayer, NNBeingTested);
	
	/*
	currentPhase = GAME_PHASE_LONGDISTANCECOMBAT;
	NNBeingTested = GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN;	
	totalError = totalError + THperformGenericCombatWithAllPermutationsOfUnitsInListAndCompareExperiencesWithNN(currentPhase, firstUnitInUnitList, currentPlayer, NNBeingTested);
		
	currentPhase = GAME_PHASE_LONGDISTANCECOMBAT;
	NNBeingTested = GAME_INDEX_OF_COMBAT_EXPERIENCE_NN;
	totalError = totalError + THperformGenericCombatWithAllPermutationsOfUnitsInListAndCompareExperiencesWithNN(currentPhase, firstUnitInUnitList, currentPlayer, NNBeingTested);
	*/

	double averageErrorPropertiesExperience = totalErrorPropertiesExperience/(pow((double)numberOfUnits, 2) * (double)numberOfNetworks * (double)numberOfScenarios);
	
	//cout << "averageErrorPropertiesExperience = " << averageErrorPropertiesExperience << endl;
	cout << "averageAccuracyPropertiesExperience = " << (1.0F-averageErrorPropertiesExperience)*100.0F << "%" <<endl;

	double averageErrorCombatExperience = totalErrorCombatExperience/(pow((double)numberOfUnits, 2) * (double)numberOfNetworks * (double)numberOfScenarios);
	
	//cout << "averageErrorCombatExperience = " << averageErrorCombatExperience << endl;
	cout << "averageAccuracyCombatExperience = " << (1.0F-averageErrorCombatExperience)*100.0F << "%" <<endl;

	
}


//THIS IS NOT FINISHED YET!
	//assume units can only have 1 part of each part type (Ie - NO UNIT GROUPS!) 
long THgenerateUnitListwithAllUnitProperties(UnitListClass * firstUnitInUnitList)
{
	bool result = true;
	long numberOfUnits = 0;
	
	UnitListClass * currentUnitInUnitList = firstUnitInUnitList;		

	
	//count number of units in each
	RulesClass * currentReferenceRulesClass;
	
	currentReferenceRulesClass = LRRCrulesUnitTypeCatagories;
	int numberUnitTypeCatagories = 0;
	while(currentReferenceRulesClass->next != NULL)
	{
		numberUnitTypeCatagories++;
		currentReferenceRulesClass = currentReferenceRulesClass->next;
	}

	currentReferenceRulesClass = LRRCrulesUnitCombatDetailsAttackCloseCombat;
	int numberUnitCombatDetailsAttackCloseCombat = 0;
	while(currentReferenceRulesClass->next != NULL)
	{
		numberUnitCombatDetailsAttackCloseCombat ++;
		currentReferenceRulesClass = currentReferenceRulesClass->next;
	}
	
	currentReferenceRulesClass = LRRCrulesUnitCombatDetailsAttackLongDistance;
	int numberUnitCombatDetailsAttackLongDistance = 0;
	while(currentReferenceRulesClass->next != NULL)
	{
		numberUnitCombatDetailsAttackLongDistance ++;
		currentReferenceRulesClass = currentReferenceRulesClass->next;
	}
	
	currentReferenceRulesClass = LRRCrulesUnitCombatDetailsDefenceHead;
	int numberUnitCombatDetailsDefenceHead = 0;
	while(currentReferenceRulesClass->next != NULL)
	{
		numberUnitCombatDetailsDefenceHead++;
		currentReferenceRulesClass = currentReferenceRulesClass->next;
	}
	
	currentReferenceRulesClass = LRRCrulesUnitCombatDetailsDefenceTorso;
	int numberUnitCombatDetailsDefenceTorso = 0;
	while(currentReferenceRulesClass->next != NULL)
	{
		numberUnitCombatDetailsDefenceTorso++;
		currentReferenceRulesClass = currentReferenceRulesClass->next;
	}
	
	currentReferenceRulesClass = LRRCrulesUnitCombatDetailsDefenceShield;
	int numberUnitCombatDetailsDefenceShield = 0;
	while(currentReferenceRulesClass->next != NULL)
	{
		numberUnitCombatDetailsDefenceShield++;
		currentReferenceRulesClass = currentReferenceRulesClass->next;
	}

	//for(int indexUnitTypeCatagories = 0; indexUnitTypeCatagories < numberUnitTypeCatagories; indexUnitTypeCatagories++)
	//{
	int indexUnitTypeCatagories = 1;	//1 = person
			
		for(int indexUnitCombatDetailsAttackCloseCombat = 0; indexUnitCombatDetailsAttackCloseCombat < numberUnitCombatDetailsAttackCloseCombat; indexUnitCombatDetailsAttackCloseCombat++)
		{
			for(int indexUnitCombatDetailsAttackLongDistance = 0; indexUnitCombatDetailsAttackLongDistance < numberUnitCombatDetailsAttackLongDistance; indexUnitCombatDetailsAttackLongDistance++)
			{
				for(int indexUnitCombatDetailsDefenceHead = 0; indexUnitCombatDetailsDefenceHead < numberUnitCombatDetailsDefenceHead; indexUnitCombatDetailsDefenceHead++)
				{
					for(int indexUnitCombatDetailsDefenceTorso = 0; indexUnitCombatDetailsDefenceTorso < numberUnitCombatDetailsDefenceTorso; indexUnitCombatDetailsDefenceTorso++)
					{
						for(int indexUnitCombatDetailsDefenceShield = 0; indexUnitCombatDetailsDefenceShield < numberUnitCombatDetailsDefenceShield; indexUnitCombatDetailsDefenceShield++)
						{
							numberOfUnits++;
							
							ModelDetails * currentUnitDetails = new ModelDetails();
							if(!generateUnitDetails(currentUnitDetails, indexUnitTypeCatagories, indexUnitCombatDetailsAttackCloseCombat, indexUnitCombatDetailsAttackLongDistance, indexUnitCombatDetailsDefenceHead, indexUnitCombatDetailsDefenceTorso, indexUnitCombatDetailsDefenceShield))
							{
								result = false;
							}
							performFinalUnitClassCalculations(currentUnitDetails);
							currentUnitInUnitList->unitDetails = currentUnitDetails;
							currentUnitInUnitList->number = numberOfUnits;	//record unit number - for debugging
							UnitListClass * newUnit = new UnitListClass();
							currentUnitInUnitList->next = newUnit;
							currentUnitInUnitList=currentUnitInUnitList->next;
						}		
					}		
				}		
			}		
		}		
	//}
	
		cout << "numberOfUnits = " << numberOfUnits << endl;
		cout << "NOT USED: numberUnitTypeCatagories = " << numberUnitTypeCatagories << endl;
		cout << "numberUnitCombatDetailsAttackCloseCombat = " << numberUnitCombatDetailsAttackCloseCombat << endl;
		cout << "numberUnitCombatDetailsAttackLongDistance = " << numberUnitCombatDetailsAttackLongDistance << endl;
		cout << "numberUnitCombatDetailsDefenceHead = " << numberUnitCombatDetailsDefenceHead << endl;
		cout << "numberUnitCombatDetailsDefenceTorso = " << numberUnitCombatDetailsDefenceTorso << endl;
		cout << "numberUnitCombatDetailsDefenceShield = " << numberUnitCombatDetailsDefenceShield << endl;
		//exit(0);
				
	return numberOfUnits;
}

bool generateUnitDetails(ModelDetails * unitDetails, int selectionUnitTypeCatagories, int selectionUnitCombatDetailsAttackCloseCombat, int selectionUnitCombatDetailsAttackLongDistance, int selectionUnitCombatDetailsDefenceHead, int selectionUnitCombatDetailsDefenceTorso, int selectionUnitCombatDetailsDefenceShield)
{
	RecordClass * currentRecord;
			
	currentRecord = unitDetails->recordOfUnitTypeDetails;
	int indexUnitTypeCatagories = 0;
	while(currentRecord->next != NULL)
	{
		if(indexUnitTypeCatagories == selectionUnitTypeCatagories)
		{
			currentRecord->numberOfThisPartIDInTheUnit = 1;
		}
		indexUnitTypeCatagories++;
		currentRecord = currentRecord->next;
	}

	currentRecord = unitDetails->recordOfUnitCombatDetailsAttackCloseCombat;
	int indexUnitCombatDetailsAttackCloseCombat = 0;
	while(currentRecord->next != NULL)
	{
		if(indexUnitCombatDetailsAttackCloseCombat == selectionUnitCombatDetailsAttackCloseCombat)
		{
			currentRecord->numberOfThisPartIDInTheUnit = 1;
		}
		indexUnitCombatDetailsAttackCloseCombat ++;
		currentRecord = currentRecord->next;
	}
	
	currentRecord = unitDetails->recordOfUnitCombatDetailsAttackLongDistance;
	int indexUnitCombatDetailsAttackLongDistance = 0;
	while(currentRecord->next != NULL)
	{
		if(indexUnitCombatDetailsAttackLongDistance == selectionUnitCombatDetailsAttackLongDistance)
		{
			currentRecord->numberOfThisPartIDInTheUnit = 1;
		}
		indexUnitCombatDetailsAttackLongDistance ++;
		currentRecord = currentRecord->next;
	}
	
	currentRecord = unitDetails->recordOfUnitCombatDetailsDefenceHead;
	int indexUnitCombatDetailsDefenceHead = 0;
	while(currentRecord->next != NULL)
	{
		if(indexUnitCombatDetailsDefenceHead == selectionUnitCombatDetailsDefenceHead)
		{
			currentRecord->numberOfThisPartIDInTheUnit = 1;
		}
		indexUnitCombatDetailsDefenceHead++;
		currentRecord = currentRecord->next;
	}
	
	currentRecord = unitDetails->recordOfUnitCombatDetailsDefenceTorso;
	int indexUnitCombatDetailsDefenceTorso = 0;
	while(currentRecord->next != NULL)
	{
		if(indexUnitCombatDetailsDefenceTorso == selectionUnitCombatDetailsDefenceTorso)
		{
			currentRecord->numberOfThisPartIDInTheUnit = 1;
		}
		indexUnitCombatDetailsDefenceTorso++;
		currentRecord = currentRecord->next;
	}
	
	currentRecord = unitDetails->recordOfUnitCombatDetailsDefenceShield;
	int indexUnitCombatDetailsDefenceShield = 0;
	while(currentRecord->next != NULL)
	{
		if(indexUnitCombatDetailsDefenceShield == selectionUnitCombatDetailsDefenceShield)
		{
			currentRecord->numberOfThisPartIDInTheUnit = 1;
		}
		indexUnitCombatDetailsDefenceShield++;
		currentRecord = currentRecord->next;
	}
	
}

bool THperformGenericCombatWithAllPermutationsOfUnitsInListAndAddExperiences(int currentPhase, UnitListClass * firstUnitInUnitList, Player * currentPlayer, int NNBeingTested)
{
	bool result = true;
	
	UnitListClass * currentUnitAttackerInList = firstUnitInUnitList;
	long unitAttackerIndex = 0;
	while(currentUnitAttackerInList -> next != NULL)
	{
		cout << "unitAttackerIndex = " << unitAttackerIndex << endl;
		
		UnitListClass * currentUnitDefenderInList = firstUnitInUnitList;
		long unitDefenderIndex = 0;
		while(currentUnitDefenderInList -> next != NULL)
		{
			THperformGenericCombatWithTwoCombatReadyUnitsAndAddExperience(currentPhase, currentUnitAttackerInList, currentUnitDefenderInList, currentPlayer, NNBeingTested);
			currentUnitDefenderInList = currentUnitDefenderInList->next;
			unitDefenderIndex++;
		}
		currentUnitAttackerInList = currentUnitAttackerInList->next;
		unitAttackerIndex++;
	}
	
	return result;
}

double THperformGenericCombatWithAllPermutationsOfUnitsInListAndCompareExperiencesWithNN(int currentPhase, UnitListClass * firstUnitInUnitList, Player * currentPlayer, int NNBeingTested)	//firstPlayerInList contains the NN
{
	//bool result = true;
	double totalError = 0.0;
				
	UnitListClass * currentUnitAttackerInList = firstUnitInUnitList;
	
	while(currentUnitAttackerInList -> next != NULL)
	{
		UnitListClass * currentUnitDefenderInList = firstUnitInUnitList;
		while(currentUnitDefenderInList -> next != NULL)
		{
			totalError = totalError + THperformGenericCombatWithTwoCombatReadyUnitsAndCompareExperienceWithNN(currentPhase, currentUnitAttackerInList, currentUnitDefenderInList, currentPlayer, NNBeingTested);
			currentUnitDefenderInList = currentUnitDefenderInList->next;
		}
		currentUnitAttackerInList = currentUnitAttackerInList->next;
	}
	
	//return result;
	return totalError;
}



		
		
		
bool THperformGenericCombatWithTwoCombatReadyUnitsAndAddExperience(int currentPhase, UnitListClass * unitAttacker, UnitListClass * unitDefender, Player * currentPlayer, int NNBeingTested)
{
	bool result = true;
	
	bool unit1TakesTheInitative = true;
	bool unit2TakesTheInitative = true;
	bool unit1intendsToPerformCloseCombatAttack = true;
	bool unit2intendsToPerformCloseCombatAttack = true;
	bool unit1performsLongDistanceAttack = true;
	bool unit2performsLongDistanceAttack = true;
	int combatResult;
	
	if(currentPhase == GAME_PHASE_CLOSECOMBAT)
	{
		combatResult = performCloseCombat(unitAttacker->unitDetails, unitDefender->unitDetails,  unit1intendsToPerformCloseCombatAttack,  unit2intendsToPerformCloseCombatAttack, unit1TakesTheInitative, unit2TakesTheInitative);
	}
	else if(currentPhase == GAME_PHASE_LONGDISTANCECOMBAT)
	{
		combatResult = performLongDistanceCombat(unitAttacker->unitDetails, unitDefender->unitDetails,  unit1performsLongDistanceAttack, unit2performsLongDistanceAttack);
	}
	else
	{
		result = false;
		cout << "error: THperformGenericCombatWithTwoCombatReadyUnitsAndAddExperience() illegal phase " << endl;
	}


	//cout << "a1" << endl;
	
	//add experience info for units

	int combatExperienceResult;
	long unitTheoreticalBestDecisionToRecord;
	if((combatResult == ATTACK_NEITHER_UNIT_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_STRIKES_UNIT2_CAN_STRIKE))
	{

	}
	else if((combatResult == ATTACK_BOTH_UNITS_STRIKE_BOTH_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_UNITS_STRIKE_BOTH_CAN_STRIKE))
	{

	}
	else if((combatResult == ATTACK_UNIT1_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT1_STRIKES_UNIT1_CAN_STRIKE))
	{
		unitTheoreticalBestDecisionToRecord = COMBAT_EXPERIENCE_OUTPUT_DECISION_ATTACK_CLOSECOMBAT;
		
		if(NNBeingTested == GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN)
		{
			if(!THaddExperienceFromUnitPropertiesDecision(unitAttacker, unitDefender, unitTheoreticalBestDecisionToRecord, currentPlayer))		//arbitrary player, OLD=defenderPlayer
			{
				result = false;
				cout << "Error: cannot addExperiencesFromUnitDecision" << endl;
			}
		}
		else if(NNBeingTested == GAME_INDEX_OF_COMBAT_EXPERIENCE_NN)
		{	
			if(!THaddExperienceFromUnitCombatDecision(unitAttacker, unitDefender, unitTheoreticalBestDecisionToRecord, currentPlayer))		//arbitrary player, OLD=defenderPlayer
			{
				result = false;
				cout << "Error: cannot addExperiencesFromUnitDecision" << endl;
			}	      
		}
		else
		{
			cout << "error: THperformGenericCombatWithTwoCombatReadyUnitsAndAddExperience() illegal NNBeingTested " << endl;
		}
	}
	else if((combatResult == ATTACK_UNIT2_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT2_STRIKES_UNIT2_CAN_STRIKE))
	{
		unitTheoreticalBestDecisionToRecord = COMBAT_EXPERIENCE_OUTPUT_DECISION_EVADE;
		
		if(NNBeingTested == GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN)
		{
			if(!THaddExperienceFromUnitPropertiesDecision(unitAttacker, unitDefender, unitTheoreticalBestDecisionToRecord, currentPlayer))		//arbitrary player, OLD=defenderPlayer
			{
				result = false;
				cout << "Error: cannot addExperiencesFromUnitDecision" << endl;
			}
		}
		else if(NNBeingTested == GAME_INDEX_OF_COMBAT_EXPERIENCE_NN)
		{	
			if(!THaddExperienceFromUnitCombatDecision(unitAttacker, unitDefender, unitTheoreticalBestDecisionToRecord, currentPlayer))		//arbitrary player, OLD=defenderPlayer
			{
				result = false;
				cout << "Error: cannot addExperiencesFromUnitDecision" << endl;
			}	      
		}
		else
		{
			cout << "error: THperformGenericCombatWithTwoCombatReadyUnitsAndAddExperience() illegal NNBeingTested " << endl;
		}
	}
	


	//cout << "a2" << endl;
	

	if((combatResult == ATTACK_NEITHER_UNIT_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_STRIKES_UNIT2_CAN_STRIKE))
	{

	}
	else if((combatResult == ATTACK_BOTH_UNITS_STRIKE_BOTH_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_UNITS_STRIKE_BOTH_CAN_STRIKE))
	{

	}
	else if((combatResult == ATTACK_UNIT2_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT2_STRIKES_UNIT2_CAN_STRIKE))
	{
		unitTheoreticalBestDecisionToRecord = COMBAT_EXPERIENCE_OUTPUT_DECISION_ATTACK_CLOSECOMBAT;
		
		if(NNBeingTested == GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN)
		{
			if(!THaddExperienceFromUnitPropertiesDecision(unitDefender, unitAttacker, unitTheoreticalBestDecisionToRecord, currentPlayer))		//arbitrary player, OLD=defenderPlayer
			{
				result = false;
				cout << "Error: cannot addExperiencesFromUnitDecision" << endl;
			}
		}
		else if(NNBeingTested == GAME_INDEX_OF_COMBAT_EXPERIENCE_NN)
		{	
			if(!THaddExperienceFromUnitCombatDecision(unitDefender, unitAttacker, unitTheoreticalBestDecisionToRecord, currentPlayer))		//arbitrary player, OLD=defenderPlayer
			{
				result = false;
				cout << "Error: cannot addExperiencesFromUnitDecision" << endl;
			}	      
		}
		else
		{
			cout << "error: THperformGenericCombatWithTwoCombatReadyUnitsAndAddExperience() illegal NNBeingTested " << endl;
		}
	}
	else if((combatResult == ATTACK_UNIT1_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT1_STRIKES_UNIT1_CAN_STRIKE))
	{
		unitTheoreticalBestDecisionToRecord = COMBAT_EXPERIENCE_OUTPUT_DECISION_EVADE;
		
		if(NNBeingTested == GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN)
		{
			if(!THaddExperienceFromUnitPropertiesDecision(unitDefender, unitAttacker, unitTheoreticalBestDecisionToRecord, currentPlayer))		//arbitrary player, OLD=defenderPlayer
			{
				result = false;
				cout << "Error: cannot addExperiencesFromUnitDecision" << endl;
			}
		}
		else if(NNBeingTested == GAME_INDEX_OF_COMBAT_EXPERIENCE_NN)
		{	
			if(!THaddExperienceFromUnitCombatDecision(unitDefender, unitAttacker, unitTheoreticalBestDecisionToRecord, currentPlayer))		//arbitrary player, OLD=defenderPlayer
			{
				result = false;
				cout << "Error: cannot addExperiencesFromUnitDecision" << endl;
			}	      
		}
		else
		{
			cout << "error: THperformGenericCombatWithTwoCombatReadyUnitsAndAddExperience() illegal NNBeingTested " << endl;
		}
	}
	
	
	//cout << "a3" << endl;

	
	return result;
}



double THperformGenericCombatWithTwoCombatReadyUnitsAndCompareExperienceWithNN(int currentPhase, UnitListClass * unitAttacker, UnitListClass * unitDefender, Player * currentPlayer, int NNBeingTested)
{
	double totalError = 0.0;
	//bool result = true;

	bool unit1TakesTheInitative = true;
	bool unit2TakesTheInitative = true;
	bool unit1intendsToPerformCloseCombatAttack = true;
	bool unit2intendsToPerformCloseCombatAttack = true;
	bool unit1performsLongDistanceAttack = true;
	bool unit2performsLongDistanceAttack = true;
	int combatResult;
	
	if(currentPhase == GAME_PHASE_CLOSECOMBAT)
	{
		combatResult = performCloseCombat(unitAttacker->unitDetails, unitDefender->unitDetails,  unit1intendsToPerformCloseCombatAttack,  unit2intendsToPerformCloseCombatAttack, unit1TakesTheInitative, unit2TakesTheInitative);
	}
	else if(currentPhase == GAME_PHASE_LONGDISTANCECOMBAT)
	{
		combatResult = performLongDistanceCombat(unitAttacker->unitDetails, unitDefender->unitDetails,  unit1performsLongDistanceAttack, unit2performsLongDistanceAttack);
	}
	else
	{
		//result = false;
		cout << "error: THperformGenericCombatWithTwoCombatReadyUnitsAndCompareExperienceWithNN() illegal phase " << endl;
	}


	//cout << "a1" << endl;
	
	//add experience info for units

	int combatExperienceResult;
	long unitTheoreticalBestDecisionToCompare;
	if((combatResult == ATTACK_NEITHER_UNIT_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_STRIKES_UNIT2_CAN_STRIKE))
	{

	}
	else if((combatResult == ATTACK_BOTH_UNITS_STRIKE_BOTH_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_UNITS_STRIKE_BOTH_CAN_STRIKE))
	{

	}
	else if((combatResult == ATTACK_UNIT1_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT1_STRIKES_UNIT1_CAN_STRIKE))
	{
		unitTheoreticalBestDecisionToCompare = COMBAT_EXPERIENCE_OUTPUT_DECISION_ATTACK_CLOSECOMBAT;
		if(NNBeingTested == GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN)
		{
			totalError = totalError + THcompareExperienceFromUnitPropertiesDecisionWithNN(unitAttacker, unitDefender, unitTheoreticalBestDecisionToCompare, currentPlayer);		//arbitrary player, OLD=attackerPlayer
		}
		else if(NNBeingTested == GAME_INDEX_OF_COMBAT_EXPERIENCE_NN)
		{		      
			totalError = totalError + THcompareExperienceFromUnitCombatDecisionWithNN(unitAttacker, unitDefender, unitTheoreticalBestDecisionToCompare, currentPlayer);		//arbitrary player, OLD=attackerPlayer
		}
		else
		{
			cout << "error: THperformGenericCombatWithTwoCombatReadyUnitsAndCompareExperienceWithNN() illegal NNBeingTested " << endl;
		}

	}
	else if((combatResult == ATTACK_UNIT2_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT2_STRIKES_UNIT2_CAN_STRIKE))
	{
		unitTheoreticalBestDecisionToCompare = COMBAT_EXPERIENCE_OUTPUT_DECISION_EVADE;
		if(NNBeingTested == GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN)
		{
			totalError = totalError + THcompareExperienceFromUnitPropertiesDecisionWithNN(unitAttacker, unitDefender, unitTheoreticalBestDecisionToCompare, currentPlayer);		//arbitrary player, OLD=attackerPlayer
		}
		else if(NNBeingTested == GAME_INDEX_OF_COMBAT_EXPERIENCE_NN)
		{		      
			totalError = totalError + THcompareExperienceFromUnitCombatDecisionWithNN(unitAttacker, unitDefender, unitTheoreticalBestDecisionToCompare, currentPlayer);		//arbitrary player, OLD=attackerPlayer
		}
		else
		{
			cout << "error: THperformGenericCombatWithTwoCombatReadyUnitsAndCompareExperienceWithNN() illegal NNBeingTested " << endl;
		}
	}
	

	
	if((combatResult == ATTACK_NEITHER_UNIT_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_STRIKES_UNIT2_CAN_STRIKE))
	{

	}
	else if((combatResult == ATTACK_BOTH_UNITS_STRIKE_BOTH_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_UNITS_STRIKE_BOTH_CAN_STRIKE))
	{

	}
	else if((combatResult == ATTACK_UNIT2_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT2_STRIKES_UNIT2_CAN_STRIKE))
	{
		unitTheoreticalBestDecisionToCompare = COMBAT_EXPERIENCE_OUTPUT_DECISION_ATTACK_CLOSECOMBAT;
		if(NNBeingTested == GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN)
		{
			totalError = totalError + THcompareExperienceFromUnitPropertiesDecisionWithNN(unitDefender, unitAttacker, unitTheoreticalBestDecisionToCompare, currentPlayer);		//arbitrary player, OLD=attackerPlayer
		}
		else if(NNBeingTested == GAME_INDEX_OF_COMBAT_EXPERIENCE_NN)
		{		      
			totalError = totalError + THcompareExperienceFromUnitCombatDecisionWithNN(unitDefender, unitAttacker, unitTheoreticalBestDecisionToCompare, currentPlayer);		//arbitrary player, OLD=attackerPlayer
		}
		else
		{
			cout << "error: THperformGenericCombatWithTwoCombatReadyUnitsAndCompareExperienceWithNN() illegal NNBeingTested " << endl;
		}
	}
	else if((combatResult == ATTACK_UNIT1_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT1_STRIKES_UNIT1_CAN_STRIKE))
	{
		unitTheoreticalBestDecisionToCompare = COMBAT_EXPERIENCE_OUTPUT_DECISION_EVADE;
		if(NNBeingTested == GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN)
		{
			totalError = totalError + THcompareExperienceFromUnitPropertiesDecisionWithNN(unitDefender, unitAttacker, unitTheoreticalBestDecisionToCompare, currentPlayer);		//arbitrary player, OLD=attackerPlayer
		}
		else if(NNBeingTested == GAME_INDEX_OF_COMBAT_EXPERIENCE_NN)
		{		      
			totalError = totalError + THcompareExperienceFromUnitCombatDecisionWithNN(unitDefender, unitAttacker, unitTheoreticalBestDecisionToCompare, currentPlayer);		//arbitrary player, OLD=attackerPlayer
		}
		else
		{
			cout << "error: THperformGenericCombatWithTwoCombatReadyUnitsAndCompareExperienceWithNN() illegal NNBeingTested " << endl;
		}
	}
	
	//return result;
	return totalError;
}



bool THaddExperienceFromUnitPropertiesDecision(UnitListClass * unit, UnitListClass * unitOpponent, long unitDecision, Player * currentPlayer)
{
	bool result = true;
	
	int nn = GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN;
	Reference * unitReference = new Reference();
	Reference * unitReferenceOpponent = new Reference();
	unitReference->subModelDetails = unit->unitDetails;
	unitReferenceOpponent->subModelDetails = unitOpponent->unitDetails;
	
#ifdef HEAPS_OF_RAM
	generateExperienceFromUnitPropertiesDecision(unitReference, unitReferenceOpponent, unitDecision, (unit->currentExperience[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN]));
	
	Experience * newExperience = new Experience();
	unit->currentExperience[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN]->next = newExperience;
	unit->currentExperience[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN] = unit->currentExperience[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN]->next;
		
#else
	Experience * newExperience = new Experience();
	generateExperienceFromUnitPropertiesDecision(unitReference, unitReferenceOpponent, unitDecision, newExperience);

	resetInputsAndClassTargets((currentPlayer->firstInputNeuronInNetwork[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN]), (currentPlayer->firstOutputNeuronInNetwork[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN]), (currentPlayer->numberOfInputNeurons[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN]), (currentPlayer->numberOfOutputNeurons[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN]), newExperience);
	double trainingErrorNotUsed = ANNBackPropogationPass(currentPlayer->firstInputNeuronInNetwork[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN], currentPlayer->firstOutputNeuronInNetwork[GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN]);

	delete newExperience;
		
#endif

	unitReference->subModelDetails = NULL;
	unitReferenceOpponent->subModelDetails = NULL;
	delete unitReference;
	delete unitReferenceOpponent;

	return result;
}

bool THaddExperienceFromUnitCombatDecision(UnitListClass * unit, UnitListClass * unitOpponent, long unitDecision, Player * currentPlayer)
{
	bool result = true;
	
	int nn = GAME_INDEX_OF_COMBAT_EXPERIENCE_NN;
	Reference * unitReference = new Reference();
	Reference * unitReferenceOpponent = new Reference();
	unitReference->subModelDetails = unit->unitDetails;
	unitReferenceOpponent->subModelDetails = unitOpponent->unitDetails;
	
#ifdef HEAPS_OF_RAM
	generateExperienceFromUnitCombatDecision(unitReference, unitReferenceOpponent, unitDecision, (unit->currentExperience[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN]));
	
	Experience * newExperience = new Experience();
	unit->currentExperience[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN]->next = newExperience;
	unit->currentExperience[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN] = unit->currentExperience[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN]->next;
		
#else
	Experience * newExperience = new Experience();
	generateExperienceFromUnitPropertiesDecision(unitReference, unitReferenceOpponent, unitDecision, newExperience);

	resetInputsAndClassTargets((currentPlayer->firstInputNeuronInNetwork[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN]), (currentPlayer->firstOutputNeuronInNetwork[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN]), (currentPlayer->numberOfInputNeurons[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN]), (currentPlayer->numberOfOutputNeurons[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN]), newExperience);
	double trainingErrorNotUsed = ANNBackPropogationPass(currentPlayer->firstInputNeuronInNetwork[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN], currentPlayer->firstOutputNeuronInNetwork[GAME_INDEX_OF_COMBAT_EXPERIENCE_NN]);

	delete newExperience;
		
#endif

	unitReference->subModelDetails = NULL;
	unitReferenceOpponent->subModelDetails = NULL;
	delete unitReference;
	delete unitReferenceOpponent;
	
	return result;
}


double THcompareExperienceFromUnitPropertiesDecisionWithNN(UnitListClass * unit, UnitListClass * unitOpponent, long unitDecision, Player * currentPlayer)
{
	bool result = true;

	//test proposed experience against network decision	//fire the current experience object inputs into the network
	Reference * unitReference = new Reference();
	Reference * unitReferenceOpponent = new Reference();
	unitReference->subModelDetails = unit->unitDetails;
	unitReferenceOpponent->subModelDetails = unitOpponent->unitDetails;
	
	int nn = GAME_INDEX_OF_PROPERTIES_EXPERIENCE_NN;
	Experience * experienceWithoutKnownOutput = new Experience[GAME_NUMBER_OF_EXPERIENCE_NN+1];
	double experienceBackPropagationPassError;
	long unitHypotheticalDecisionToTest;
	unitHypotheticalDecisionToTest = unitDecision;
		//generate proposed experience
	generateExperienceFromUnitPropertiesDecision(unitReference, unitReferenceOpponent, unitHypotheticalDecisionToTest, &(experienceWithoutKnownOutput[nn]));
	experienceBackPropagationPassError = calculateExperienceErrorForHypotheticalDecision(currentPlayer->firstInputNeuronInNetwork[nn], currentPlayer->firstOutputNeuronInNetwork[nn], currentPlayer->numberOfInputNeurons[nn], currentPlayer->numberOfOutputNeurons[nn], &(experienceWithoutKnownOutput[nn]));

	unitReference->subModelDetails = NULL;
	unitReferenceOpponent->subModelDetails = NULL;
	delete unitReference;
	delete unitReferenceOpponent;
	
	return experienceBackPropagationPassError;
}


double THcompareExperienceFromUnitCombatDecisionWithNN(UnitListClass * unit, UnitListClass * unitOpponent, long unitDecision, Player * currentPlayer)
{
	bool result = true;

	//test proposed experience against network decision	//fire the current experience object inputs into the network
	Reference * unitReference = new Reference();
	Reference * unitReferenceOpponent = new Reference();
	unitReference->subModelDetails = unit->unitDetails;
	unitReferenceOpponent->subModelDetails = unitOpponent->unitDetails;
	
	int nn = GAME_INDEX_OF_COMBAT_EXPERIENCE_NN;
	Experience * experienceWithoutKnownOutput = new Experience[GAME_NUMBER_OF_EXPERIENCE_NN+1];
	double experienceBackPropagationPassError;
	long unitHypotheticalDecisionToTest;
	unitHypotheticalDecisionToTest = unitDecision;
		//generate proposed experience
	generateExperienceFromUnitCombatDecision(unitReference, unitReferenceOpponent, unitHypotheticalDecisionToTest, &(experienceWithoutKnownOutput[nn]));
	experienceBackPropagationPassError = calculateExperienceErrorForHypotheticalDecision(currentPlayer->firstInputNeuronInNetwork[nn], currentPlayer->firstOutputNeuronInNetwork[nn], currentPlayer->numberOfInputNeurons[nn], currentPlayer->numberOfOutputNeurons[nn], &(experienceWithoutKnownOutput[nn]));

	unitReference->subModelDetails = NULL;
	unitReferenceOpponent->subModelDetails = NULL;
	delete unitReference;
	delete unitReferenceOpponent;
	
	return experienceBackPropagationPassError;
}








/*
double experienceBackPropagationPassError;
int nn;
long unitHypotheticalDecisionToTest;
*/



/*
nn = GAME_INDEX_OF_GLOBAL_EXPERIENCE_NN;
generateExperienceFromGlobalDecision(currentPlayer->firstUnitInUnitList, initialReferenceInThisPhaseStartScene, referenceToPlayerUnitThatIsFindingAnOpponent, referenceForOpposingPlayerCurrentUnit, &(experienceWithoutKnownOutput[nn]));
experienceBackPropagationPassError = calculateExperienceErrorForHypotheticalDecision(currentPlayer->firstInputNeuronInNetwork[nn], currentPlayer->firstOutputNeuronInNetwork[nn], currentPlayer->numberOfInputNeurons[nn], currentPlayer->numberOfOutputNeurons[nn], &(experienceWithoutKnownOutput[nn]));
currentUnit->selfLearningTempVarBackPropagationPassError[nn] = experienceBackPropagationPassError;

delete[] experienceWithoutKnownOutput;
delete referenceForOpposingPlayerCurrentUnit;
//cout << "here3e" << endl;			
*/





#endif





